#############################################################################
# syslink.mak                                                               #
#                                                                           #
# Targets to check, configure and build SysLink components                  #
#############################################################################
#
#
#############################################################################
#                                                                           #
#   Copyright (C) 2010 Texas Instruments Incorporated                       #
#     http://www.ti.com/                                                    #
#                                                                           #
#############################################################################
#
#
#############################################################################
#                                                                           #
#  Redistribution and use in source and binary forms, with or without       #
#  modification, are permitted provided that the following conditions       #
#  are met:                                                                 #
#                                                                           #
#    Redistributions of source code must retain the above copyright         #
#    notice, this list of conditions and the following disclaimer.          #
#                                                                           #
#    Redistributions in binary form must reproduce the above copyright      #
#    notice, this list of conditions and the following disclaimer in the    #
#    documentation and/or other materials provided with the                 #
#    distribution.                                                          #
#                                                                           #
#    Neither the name of Texas Instruments Incorporated nor the names of    #
#    its contributors may be used to endorse or promote products derived    #
#    from this software without specific prior written permission.          #
#                                                                           #
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS      #
#  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT        #
#  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR    #
#  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT     #
#  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    #
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT         #
#  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,    #
#  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY    #
#  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT      #
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE    #
#  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.     #
#                                                                           #
#############################################################################


#   ----------------------------------------------------------------------------
#   Include the top-level Rules.mak file
#   ----------------------------------------------------------------------------
include ../config.mak
include ../Rules.mak


#   ----------------------------------------------------------------------------
#   SysLink directory definition
#   ----------------------------------------------------------------------------
SYSLINK_ROOT := $(SYSLINK_INSTALL_DIR)
#SYSLINK_PLATFORM := $(SYSLINK_PLATFORM)
ifndef SYSLINK_VARIANT
SYSLINK_VARIANT := $(SYSLINK_PLATFORM)
endif
ifndef SYSLINK_LOADER
SYSLINK_LOADER  := COFF
endif

PLATFORMDIR := $(C6RUN_INSTALL_DIR)/platforms/$(PLATFORM)

MOD_PATH := $(SYSLINK_ROOT)/ti/syslink/bin/$(SYSLINK_VARIANT)
MOD_NAME := syslink.ko

#   ----------------------------------------------------------------------------
#   Compiler and Linker procedure
#   ----------------------------------------------------------------------------
.PHONY:	all gpp_libs gpp_samples dsp_libs kernel_mod install_kernel_mod

# The default build target.
all: gpp_libs dsp_libs kernel_mod


#   ----------------------------------------------------------------------------
#   Build the gpp libraries
#   ----------------------------------------------------------------------------
gpp_libs:
	@echo -n "Building SysLink GPP-side userspace libraries (be patient)..."
	@$(XDC_INSTALL_DIR)/gmake -C $(SYSLINK_ROOT)/ti/syslink/utils/hlos/usr/Linux \
          SYSLINK_ROOT=$(SYSLINK_ROOT) \
          SYSLINK_PLATFORM=$(SYSLINK_PLATFORM) \
          SYSLINK_VARIANT=$(SYSLINK_VARIANT) \
          TOOLCHAIN_PREFIX=$(ARM_TOOLCHAIN_PATH)/bin/$(ARM_TOOLCHAIN_PREFIX) \
          SYSLINK_PKGPATH="$(IPC_INSTALL_DIR)/packages;$(SYSLINK_ROOT)" \
          release debug
	@echo "complete!"

gpp_samples:
	@echo -n "Building SysLink GPP-side userspace examples (be patient)..."
	@for MOD in procMgr notify messageQ listMP heapBufMP heapMemMP gateMP frameq ringIO ringIO_gpp sharedRegion; do \
    $(XDC_INSTALL_DIR)/gmake -C $(SYSLINK_ROOT)/ti/syslink/samples/hlos/$$MOD/usr/Linux \
          SYSLINK_ROOT=$(SYSLINK_ROOT) \
          SYSLINK_PLATFORM=$(SYSLINK_PLATFORM) \
          SYSLINK_VARIANT=$(SYSLINK_VARIANT) \
          TOOLCHAIN_PREFIX=$(ARM_TOOLCHAIN_PATH)/bin/$(ARM_TOOLCHAIN_PREFIX) \
          SYSLINK_PKGPATH="$(IPC_INSTALL_DIR)/packages;$(SYSLINK_ROOT)"; \
  done
	@echo "complete!"

#   ----------------------------------------------------------------------------
#   Build the dsp libraries
#   ----------------------------------------------------------------------------
dsp_libs:
	@echo -n "Building SysLink DSP-side libraries (be patient)..."
	@cd $(SYSLINK_ROOT)/ti; \
    CODEGEN_INSTALL_DIR=$(CODEGEN_INSTALL_DIR) \
    SYSLINK_PLATFORM=$(SYSLINK_PLATFORM) \
    SYSLINK_VARIANT=$(SYSLINK_VARIANT) \
    $(XDC_INSTALL_DIR)/xdc XDCPATH="$(IPC_INSTALL_DIR)/packages;$(SYSBIOS_INSTALL_DIR)/packages" XDCBUILDCFG="$(PLATFORMDIR)/platform_syslink.bld" -PR 'xdcpkg syslink | grep -v /samples';
	@echo "complete!"

dsp_samples:
	@echo -n "Building SysLink DSP-side examples (be patient)..."
	@cd $(SYSLINK_ROOT)/ti/syslink/ipc; \
    CODEGEN_INSTALL_DIR=$(CODEGEN_INSTALL_DIR) \
    SYSLINK_PLATFORM=$(SYSLINK_PLATFORM) \
    SYSLINK_VARIANT=$(SYSLINK_VARIANT) \
    $(XDC_INSTALL_DIR)/xdc all XDCARGS="profile=debug" XDCPATH="$(IPC_INSTALL_DIR)/packages;$(SYSBIOS_INSTALL_DIR)/packages" XDCBUILDCFG="$(PLATFORMDIR)/platform_syslink.bld" -PR .;
	@cd $(SYSLINK_ROOT)/ti/syslink/samples/rtos; \
    CODEGEN_INSTALL_DIR=$(CODEGEN_INSTALL_DIR) \
    SYSLINK_PLATFORM=$(SYSLINK_PLATFORM) \
    SYSLINK_VARIANT=$(SYSLINK_VARIANT) \
    $(XDC_INSTALL_DIR)/xdc all XDCARGS="profile=debug" XDCPATH="$(IPC_INSTALL_DIR)/packages;$(SYSBIOS_INSTALL_DIR)/packages" XDCBUILDCFG="$(PLATFORMDIR)/platform_syslink.bld" -PR .;


#   ----------------------------------------------------------------------------
#   Build the DSPLink kernel module for the configured platform, and make sure the
#   kernel_binaries directory is kept in sync. 
#   ----------------------------------------------------------------------------
kernel_mod:
	@echo -n "Building SysLink kernel module..."
	$(XDC_INSTALL_DIR)/gmake -C $(SYSLINK_ROOT)/ti/syslink/utils/hlos/knl/Linux \
          ARCH=arm \
          SYSLINK_ROOT=$(SYSLINK_ROOT) \
          SYSLINK_PLATFORM=$(SYSLINK_PLATFORM) \
          SYSLINK_VARIANT=$(SYSLINK_VARIANT) \
          SYSLINK_LOADER=$(SYSLINK_LOADER) \
          CROSS_COMPILE=$(ARM_TOOLCHAIN_PATH)/bin/$(ARM_TOOLCHAIN_PREFIX) \
          SYSLINK_PKGPATH="$(IPC_INSTALL_DIR)/packages;$(SYSLINK_ROOT)" \
          KDIR=$(LINUXKERNEL_INSTALL_DIR) \
          default
	@echo "complete!"

install_kernel_mod:
ifdef INSTALL_DIR
	@if [ -e $(MOD_PATH)/$(MOD_NAME) ]; then \
    echo "SysLink kernel module found. Copying to install directory..."; \
    install -m 755 $(MOD_PATH)/$(MOD_NAME) $(INSTALL_DIR)/; \
  fi
else
	@echo "You did not specify a valid INSTALL_DIR variable when running 'make install_kernel_mod'."
endif


#   ----------------------------------------------------------------------------
#   Clean Rule(s)
#   ----------------------------------------------------------------------------
.PHONY: clean gpp_libs_clean gpp_samples_clean dsp_libs_clean kernel_mod_clean
clean: gpp_libs_clean dsp_libs_clean kernel_mod_clean

gpp_libs_clean:
	@echo -n "Peforming clean on SysLink GPP-side user-space libraries..."
	@$(XDC_INSTALL_DIR)/gmake -C $(SYSLINK_ROOT)/ti/syslink/utils/hlos/usr/Linux \
          SYSLINK_ROOT=$(SYSLINK_ROOT) \
          SYSLINK_PLATFORM=$(SYSLINK_PLATFORM) \
          cleanall
	@echo "complete!"

gpp_samples_clean:
	@echo -n "Cleaning SysLink GPP-side userspace examples (be patient)..."
	@for MOD in procMgr notify messageQ listMP heapBufMP heapMemMP gateMP frameq ringIO ringIO_gpp sharedRegion; do \
    $(XDC_INSTALL_DIR)/gmake -C $(SYSLINK_ROOT)/ti/syslink/samples/hlos/$$MOD/usr/Linux \
          SYSLINK_ROOT=$(SYSLINK_ROOT) \
          SYSLINK_PLATFORM=$(SYSLINK_PLATFORM) \
          TOOLCHAIN_PREFIX=$(ARM_TOOLCHAIN_PATH)/bin/$(ARM_TOOLCHAIN_PREFIX) \
          SYSLINK_PKGPATH="$(IPC_INSTALL_DIR)/packages;$(SYSLINK_ROOT)" clean; \
  done
	@echo "complete!"

dsp_libs_clean:
	@echo -n "Peforming clean on SysLink GPP-side libraries..."
	@cd $(SYSLINK_ROOT)/ti/syslink; \
    SYSLINK_PLATFORM=$(SYSLINK_PLATFORM) $(XDC_INSTALL_DIR)/xdc clean XDCPATH="$(IPC_INSTALL_DIR)/packages;$(SYSBIOS_INSTALL_DIR)/packages" XDCBUILDCFG="$(PLATFORMDIR)/platform_syslink.bld" -PR 'xdcpkg syslink | grep -v /samples';
	@echo "complete!"

dsp_samples_clean:
	@echo -n "Cleaning SysLink DSP-side userspace examples (be patient)..."
	@cd $(SYSLINK_ROOT)/ti/syslink/ipc; \
    SYSLINK_PLATFORM=$(SYSLINK_PLATFORM) $(XDC_INSTALL_DIR)/xdc clean XDCPATH="$(IPC_INSTALL_DIR)/packages;$(SYSBIOS_INSTALL_DIR)/packages" XDCBUILDCFG="$(PLATFORMDIR)/platform_syslink.bld" -PR .;
	@cd $(SYSLINK_ROOT)/ti/syslink/samples/rtos; \
    SYSLINK_PLATFORM=$(SYSLINK_PLATFORM) $(XDC_INSTALL_DIR)/xdc clean XDCPATH="$(IPC_INSTALL_DIR)/packages;$(SYSBIOS_INSTALL_DIR)/packages" XDCBUILDCFG="$(PLATFORMDIR)/platform_syslink.bld" -PR .;
	@echo "complete!"

kernel_mod_clean:
	@echo -n "Cleaning up SysLink kernel module..."
	@$(XDC_INSTALL_DIR)/gmake -C $(SYSLINK_ROOT)/ti/syslink/utils/hlos/knl/Linux \
          ARCH=arm \
          SYSLINK_ROOT=$(SYSLINK_ROOT) \
          SYSLINK_PLATFORM=$(SYSLINK_PLATFORM) \
          CROSS_COMPILE=$(ARM_TOOLCHAIN_PATH)/bin/$(ARM_TOOLCHAIN_PREFIX) \
          SYSLINK_PKGPATH="$(IPC_INSTALL_DIR)/packages;$(SYSLINK_ROOT)" \
          KDIR=$(LINUXKERNEL_INSTALL_DIR) \
          clean
	@echo "complete!"
